-------------------------------------------------------------------------------
-- Locals                                                                    --
-------------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("DeuceLog")
local tourist = AceLibrary("Tourist-2.0")
local parser = ParserLib:GetInstance("1.1")

local lastDamageDealt = nil
local lastDamageRecieved = nil

local temporaryGuildList = nil

local playerName = nil
local lastGuildIndex = nil
local lastDing = nil
local lossThrottle = nil
local winsThrottle = nil

-------------------------------------------------------------------------------
-- Localization                                                              --
-------------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {

	["DeuceLog Enabled!"] = true,
	["DeuceLog Disabled!"] = true,

	["Search"] = true,
	["Search the log database for the given character or guild."] = true,
	["<character> or <guild name>"] = true,
	["No results found for '%s'."] = true,

	["%s <%s> %d/%d (%d/%d)"] = true,
	["%s %d/%d"] = true,

	["Killed by deucebag: %s."] = true,
	["Deucebag killed: %s."] = true,

} end)

-------------------------------------------------------------------------------
-- Addon declaration                                                         --
-------------------------------------------------------------------------------

DeuceLog = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0")
DeuceLog:RegisterDB("DeuceLogDB", "DeuceLogPerCharDB")
DeuceLog:RegisterDefaults("profile", {
	enabled = true,
	guilds = {},
})
DeuceLog:RegisterDefaults("char", {
	chars = {},
	guilds = {},
})
DeuceLog:RegisterChatCommand({"/deucelog", "/dl"}, {
	type = "group",
	args = {
		search = {
			type = "text",
			name = L["Search"],
			desc = L["Search the log database for the given character or guild."],
			usage = L["<character> or <guild name>"],
			get = false,
			set = function(v) DeuceLog:Search(v) end,
		},
	},
}, "DEUCELOG")

-------------------------------------------------------------------------------
-- Initialization                                                            --
-------------------------------------------------------------------------------

function DeuceLog:OnEnable()
	playerName = UnitName("player")

	-- Find the last guild index so we can increment it when we find a new
	-- guild.
	if type(table.maxn) == "function" then
		lastGuildIndex = table.maxn(self.db.profile.guilds)
	else
		lastGuildIndex = table.getn(self.db.profile.guilds)
	end

	-- We disable when entering an instance and such.
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:ZONE_CHANGED_NEW_AREA()
end

function DeuceLog:RegisterAllEvents()
	if self:IsEventRegistered("PLAYER_TARGET_CHANGED") then return end

	-- Reports with a DING sound and some on-screen text if we
	-- mouse over or target someone that we have data for.
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "ReportTarget")
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "MouseOverTarget")

	-- respond when player dies
	self:RegisterEvent("PLAYER_DEAD")

	parser:RegisterEvent("DeuceLog", "CHAT_MSG_COMBAT_FRIENDLY_DEATH", DeuceLog.ParseEvent)
	parser:RegisterEvent("DeuceLog", "CHAT_MSG_COMBAT_HONOR_GAIN", DeuceLog.ParseEvent)
	parser:RegisterEvent("DeuceLog", "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS", DeuceLog.ParseEvent)
	parser:RegisterEvent("DeuceLog", "CHAT_MSG_COMBAT_HOSTILE_DEATH", DeuceLog.ParseEvent)
	parser:RegisterEvent("DeuceLog", "CHAT_MSG_COMBAT_PET_HITS", DeuceLog.ParseEvent)
	parser:RegisterEvent("DeuceLog", "CHAT_MSG_COMBAT_SELF_HITS", DeuceLog.ParseEvent)
	parser:RegisterEvent("DeuceLog", "CHAT_MSG_SPELL_DAMAGESHIELDS_ON_SELF", DeuceLog.ParseEvent)
	parser:RegisterEvent("DeuceLog", "CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE", DeuceLog.ParseEvent)
	parser:RegisterEvent("DeuceLog", "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE", DeuceLog.ParseEvent)
	parser:RegisterEvent("DeuceLog", "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE", DeuceLog.ParseEvent)
	parser:RegisterEvent("DeuceLog", "CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE", DeuceLog.ParseEvent)
	parser:RegisterEvent("DeuceLog", "CHAT_MSG_SPELL_PET_DAMAGE", DeuceLog.ParseEvent)
	parser:RegisterEvent("DeuceLog", "CHAT_MSG_SPELL_SELF_DAMAGE", DeuceLog.ParseEvent)

	self:Print(L["DeuceLog Enabled!"])
end

function DeuceLog:UnregisterAllEvents()
	if not self:IsEventRegistered("PLAYER_TARGET_CHANGED") then return end

	self:UnregisterEvent("PLAYER_TARGET_CHANGED")
	self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
	self:UnregisterEvent("PLAYER_DEAD")

	parser:UnregisterAllEvents("DeuceLog")

	self:Print(L["DeuceLog Disabled!"])
end

-------------------------------------------------------------------------------
-- Utility                                                                   --
-------------------------------------------------------------------------------

function DeuceLog:Search(nameOrGuild)
	for k, v in pairs(self.db.char.chars) do
		if k == nameOrGuild then
			self:Print(string.format(L["%s %d/%d"], k, self.db.char.chars[k].wins or 0, self.db.char.chars[k].loss or 0))
			return
		end
	end

	local guildIndex = self:GetGuildIndex(nameOrGuild)
	if guildIndex ~= nil then
		if self.db.profile.guilds[guildIndex] then
			self:Print(string.format(L["%s %d/%d"], k, self.db.profile.guilds[guildIndex].wins or 0, self.db.profile.guilds[guildIndex].loss or 0))
			return
		end
	end
	-- No exact match found, try string indexing.
	for k, v in pairs(self.db.char.chars) do
		if string.find(string.lower(k), string.lower(nameOrGuild)) then
			self:Print(string.format(L["%s %d/%d"], k, self.db.char.chars[k].wins or 0, self.db.char.chars[k].loss or 0))
			return
		end
	end

	local guildIndex = nil
	local guildName = nil
	for k, name in pairs(self.db.profile.guilds) do
		if string.find(string.lower(name), string.lower(nameOrGuild)) then
			guildIndex = k
			guildName = name
			break
		end
	end
	if guildIndex ~= nil and guildName ~= nil then
		if self.db.profile.guilds[guildIndex] then
			self:Print(string.format(L["%s %d/%d"], guildName, self.db.profile.guilds[guildIndex].wins or 0, self.db.profile.guilds[guildIndex].loss or 0))
			return
		end
	end

	self:Print(string.format(L["No results found for '%s'."], nameOrGuild))
end

function DeuceLog:GetReportText( targetName, guildName )
	local guildIndex = self:GetGuildIndex( guildName )
	local wins = self.db.char.chars[targetName].wins and self.db.char.chars[targetName].wins or 0
	local loss = self.db.char.chars[targetName].loss and self.db.char.chars[targetName].loss or 0

	if guildIndex then
		local translate = L["%s <%s> %d/%d (%d/%d)"] -- target, guild, wins/losses, gwins/glosses
		local gwins = self.db.char.guilds[guildIndex].wins and self.db.char.guilds[guildIndex].wins or 0
		local gloss = self.db.char.guilds[guildIndex].loss and self.db.char.guilds[guildIndex].loss or 0
	
		return string.format(translate, targetName, guildName, wins, loss, gwins, gloss)
	else
		local translateNoGuild = L["%s %d/%d"] -- target, wins/losses
		return string.format(translateNoGuild, targetName, wins, loss)
	end
end

function DeuceLog:GetGuildIndex( guildName )
	if not guildName then return nil end
	for k, name in pairs(self.db.profile.guilds) do
		if name == guildName then
			return k
		end
	end
	return nil
end

function DeuceLog:RecordWin( targetName )
	if not targetName then return end
	if winsThrottle[targetName] and ((winsThrottle[targetName] + 10) > GetTime()) then return end
	winsThrottle[targetName] = GetTime()

	if UnitExists("target") and
		((UnitName("target") == targetName and not UnitIsEnemy("player", "target")) or
		not UnitIsPlayer("target")) then
		return
	end

	if temporaryGuildList[targetName] then
		local guildName = temporaryGuildList[targetName]
		local guildIndex = self:GetGuildIndex(guildName)
		if not guildIndex then
			table.insert(self.db.profile.guilds, guildName)
			guildIndex = self:GetGuildIndex(guildName)
		end
		if guildIndex then
			if not self.db.char.guilds[guildIndex] then self.db.char.guilds[guildIndex] = {} end
			if not self.db.char.guilds[guildIndex].wins then
				self.db.char.guilds[guildIndex].wins = 1
			else
				self.db.char.guilds[guildIndex].wins = self.db.char.guilds[guildIndex].wins + 1
			end
		end
	end
	if not self.db.char.chars[targetName] then self.db.char.chars[targetName] = {} end
	if not self.db.char.chars[targetName].wins then
		self.db.char.chars[targetName].wins = 1
	else
		self.db.char.chars[targetName].wins = self.db.char.chars[targetName].wins + 1
	end
	self:Print(string.format(L["Deucebag killed: %s."], targetName))
end

function DeuceLog:RecordLoss( targetName )
	if not targetName then return end
	if lossThrottle[targetName] and ((lossThrottle[targetName] + 10) > GetTime()) then return end
	lossThrottle[targetName] = GetTime()

	if UnitExists("target") and
		((UnitName("target") == targetName and not UnitIsEnemy("player", "target")) or
		not UnitIsPlayer("target")) then
		return
	end

	if temporaryGuildList[targetName] then
		local guildName = temporaryGuildList[targetName]
		local guildIndex = self:GetGuildIndex(guildName)
		if not guildIndex then
			table.insert(self.db.profile.guilds, guildName)
			guildIndex = self:GetGuildIndex(guildName)
		end
		if guildIndex then
			if not self.db.char.guilds[guildIndex] then self.db.char.guilds[guildIndex] = {} end
			if not self.db.char.guilds[guildIndex].loss then
				self.db.char.guilds[guildIndex].loss = 1
			else
				self.db.char.guilds[guildIndex].loss = self.db.char.guilds[guildIndex].loss + 1
			end
		end
	end
	if not self.db.char.chars[targetName] then self.db.char.chars[targetName] = {} end
	if not self.db.char.chars[targetName].loss then
		self.db.char.chars[targetName].loss = 1
	else
		self.db.char.chars[targetName].loss = self.db.char.chars[targetName].loss + 1
	end
	self:Print(string.format(L["Killed by deucebag: %s."], targetName))
end

-------------------------------------------------------------------------------
-- Events                                                                    --
-------------------------------------------------------------------------------

function DeuceLog:ParseEvent( info )
	-- Invoked from parserlib, no self reference.
	local self = DeuceLog

	if info.type == "hit" and info.source then
		if info.source == ParserLib_SELF then
			lastDamageDealt = info.victim
		elseif info.victim == ParserLib_SELF then
			lastDamageRecieved = info.source
		end
	elseif info.type == "death" then
		if info.victim == playerName then
			self:PLAYER_DEAD(info.source)
		elseif info.victim == lastDamageDealt then
			-- We killed someone
			self:RecordWin(info.victim)
		end
	elseif info.type == "honor" and info.source then
		self:RecordWin(info.source)
	end
end

function DeuceLog:ZONE_CHANGED_NEW_AREA()
	temporaryGuildList = {}
	lastDing = {}
	lossThrottle = {}
	winsThrottle = {}

	if tourist:IsBattleground(GetZoneText()) or not tourist:IsInstance(GetZoneText()) then
		-- Check that we are enabled.
		self:RegisterAllEvents()
	else
		-- Disable - we're in an instance.
		self:UnregisterAllEvents()
	end
end

-- Something killed us! REPORT!
function DeuceLog:PLAYER_DEAD(source)
	if not lastDamageRecieved or lastDamageRecieved == "" then
		if not source or source == "" then return end
		lastDamageRecieved = source
	end

	self:RecordLoss(lastDamageRecieved)
end

-- We mouse over someone, lets see if we should make a DING sound or not.
function DeuceLog:ReportTarget()
	local target = nil
	if UnitExists("mouseover") then
		target = "mouseover"
	else
		target = "target"
	end

	if	UnitExists(target) and
		UnitIsEnemy("player", target) and
		UnitIsPlayer(target) then
	
		local name = UnitName(target)
		local guildName, _, _ = GetGuildInfo(target)
		temporaryGuildList[name] = guildName

		if self.db.char.chars[name] then
			if not lastDing[name] or (lastDing[name] and (lastDing[name] + 120) < GetTime()) then
				local text = self:GetReportText(name, guildName)

				if SCT and SCT_MSG_FRAME then -- SCT 5.x
					SCT_MSG_FRAME:AddMessage(text, 0.0, 1.0, 0.0, 1)
				elseif MikSBT then
					MikSBT.DisplayMessage(text, MikSBT.DISPLAYTYPE_NOTIFICATION, false, 0, 255, 0)
				elseif CombatText_AddMessage then -- Blizzards FCT
					CombatText_AddMessage(text, COMBAT_TEXT_SCROLL_FUNCTION, 0.0, 1.0, 0.0, "sticky", nil)
				else
					UIErrorsFrame:AddMessage(text, 0.0, 1.0, 0.0, 1, UIERRORS_HOLD_TIME)

				end
				PlaySound("AuctionWindowOpen")
				lastDing[name] = GetTime()
			end
		end
	end
end

-- Credits to phyber (questsfu) for most of the tooltip code.
local _G = getfenv(0)
function DeuceLog:MouseOverTarget()
	self:ReportTarget()
	if not UnitIsEnemy("player", "mouseover") or not UnitIsPlayer("mouseover") then return end
	local tooltip = _G["GameTooltip"]
	if not tooltip or type(tooltip.GetName) ~= "function" or type(tooltip.IsVisible) ~= "function" or type(tooltip.GetText) ~= "function" then return end
	if not tooltip:IsVisible() or tooltip:GetText() == nil then return end
	local name = UnitName("mouseover")
	local guildName, _, _ = GetGuildInfo("mouseover")
	local guildIndex = self:GetGuildIndex(guildName)
	if self.db.char.chars[name] or guildIndex ~= nil then
		local text = self:GetReportText(name, guildName)
		tooltip:AddLine(text, 0.0, 1.0, 0.0)
		length = _G[tooltip:GetName() .. "TextLeft" .. tooltip:NumLines()]:GetStringWidth()
		length = length + 22
		tooltip:SetHeight(tooltip:GetHeight() + 14)
		if length > tooltip:GetWidth() then
			tooltip:SetWidth(length)
		end
	end
end

-------------------------------------------------------------------------------
-- DeuceLog API                                                              --
-------------------------------------------------------------------------------


